@@ -1,4 +1,4 @@
-"""
+'''
 Pymodbus: Modbus Protocol Implementation
 -----------------------------------------
 
@@ -9,24 +9,23 @@ TwistedModbus is built on top of the code developed by:
     Hynek Petrak <hynek@swac.cz>
 
 Released under the the BSD license
-"""
+'''
 
-from pymodbus.version import _version
-__version__ = _version.short()
+import pymodbus.version as __version
+__version__ = __version.version.short()
 __author__  = 'Galen Collins'
-
+__maintainer__ = 'dhoomakethu'
 
 #---------------------------------------------------------------------------#
 # Block unhandled logging
 #---------------------------------------------------------------------------#
-import logging
+import logging as __logging
 try:
-    from logging import NullHandler
+    from logging import NullHandler as __null
 except ImportError:
-    class NullHandler(logging.Handler):
+    class __null(__logging.Handler):
         def emit(self, record):
             pass
 
-h = NullHandler()
-logging.getLogger(__name__).addHandler(h)
+__logging.getLogger(__name__).addHandler(__null())
 
