@@ -1,4 +1,5 @@
 import socket
+import select
 import serial
 import time
 import sys
@@ -123,6 +124,10 @@ class BaseModbusClient(ModbusClientMixin):
         self.close()
 
     def idle_time(self):
+        """
+        Bus Idle Time to initiate next transaction
+        :return: time stamp
+        """
         if self.last_frame_end is None or self.silent_interval is None:
             return 0
         return self.last_frame_end + self.silent_interval
@@ -230,34 +235,43 @@ class ModbusTcpClient(BaseModbusClient):
         """
         if not self.socket:
             raise ConnectionException(self.__str__())
-            # socket.recv(size) waits until it gets some data from the host but
-            # not necessarily the entire response that can be fragmented in
-            # many packets.
-            # To avoid the splitted responses to be recognized as invalid
-            # messages and to be discarded, loops socket.recv until full data
-            # is received or timeout is expired.
-            # If timeout expires returns the read data, also if its length is
-            # less than the expected size.
+
+        # socket.recv(size) waits until it gets some data from the host but
+        # not necessarily the entire response that can be fragmented in
+        # many packets.
+        # To avoid the splitted responses to be recognized as invalid
+        # messages and to be discarded, loops socket.recv until full data
+        # is received or timeout is expired.
+        # If timeout expires returns the read data, also if its length is
+        # less than the expected size.
         self.socket.setblocking(0)
-        begin = time.time()
+
+        timeout = self.timeout
+
+        # If size isn't specified read 1 byte at a time.
+        if size is None:
+            recv_size = 1
+        else:
+            recv_size = size
 
         data = b''
-        if size is not None:
-            while len(data) < size:
-                try:
-                    data += self.socket.recv(size - len(data))
-                except socket.error:
-                    pass
-                if not self.timeout or (time.time() - begin > self.timeout):
-                    break
-        else:
-            while True:
-                try:
-                    data += self.socket.recv(1)
-                except socket.error:
-                    pass
-                if not self.timeout or (time.time() - begin > self.timeout):
-                    break
+        begin = time.time()
+        while recv_size > 0:
+            ready = select.select([self.socket], [], [], timeout)
+            if ready[0]:
+                data += self.socket.recv(recv_size)
+
+            # If size isn't specified continue to read until timeout expires.
+            if size:
+                recv_size = size - len(data)
+
+            # Timeout is reduced also if some data has been received in order
+            # to avoid infinite loops when there isn't an expected response size
+            # and the slave sends noisy data continuosly.
+            timeout -= time.time() - begin
+            if timeout <= 0:
+                break
+
         return data
 
     def is_socket_open(self):
@@ -276,11 +290,11 @@ class ModbusTcpClient(BaseModbusClient):
             "port={self.port}, timeout={self.timeout}>"
         ).format(self.__class__.__name__, hex(id(self)), self=self)
 
-
-
 # --------------------------------------------------------------------------- #
 # Modbus UDP Client Transport Implementation
 # --------------------------------------------------------------------------- #
+
+
 class ModbusUdpClient(BaseModbusClient):
     """ Implementation of a modbus udp client
     """
@@ -376,10 +390,14 @@ class ModbusUdpClient(BaseModbusClient):
 # --------------------------------------------------------------------------- #
 # Modbus Serial Client Transport Implementation
 # --------------------------------------------------------------------------- #
+
+
 class ModbusSerialClient(BaseModbusClient):
     """ Implementation of a modbus serial client
     """
     state = ModbusTransactionState.IDLE
+    inter_char_timeout = 0
+    silent_interval = 0
 
     def __init__(self, method='ascii', **kwargs):
         """ Initialize a serial client instance
@@ -414,7 +432,9 @@ class ModbusSerialClient(BaseModbusClient):
             if self.baudrate > 19200:
                 self.silent_interval = 1.75 / 1000  # ms
             else:
-                self.silent_interval = 3.5 * (1 + 8 + 2) / self.baudrate
+                self._t0 = float((1 + 8 + 2)) / self.baudrate
+                self.inter_char_timeout = 1.5 * self._t0
+                self.silent_interval = 3.5 * self._t0
             self.silent_interval = round(self.silent_interval, 6)
 
     @staticmethod
@@ -453,6 +473,7 @@ class ModbusSerialClient(BaseModbusClient):
             _logger.error(msg)
             self.close()
         if self.method == "rtu":
+            self.socket.interCharTimeout = self.inter_char_timeout
             self.last_frame_end = None
         return self.socket is not None
 
@@ -502,15 +523,22 @@ class ModbusSerialClient(BaseModbusClient):
         return 0
 
     def _wait_for_data(self):
+        size = 0
+        more_data = False
         if self.timeout is not None and self.timeout != 0:
-            condition = partial(lambda start, timeout: (time.time() - start) <= timeout, timeout=self.timeout)
+            condition = partial(lambda start, timeout:
+                                (time.time() - start) <= timeout,
+                                timeout=self.timeout)
         else:
             condition = partial(lambda dummy1, dummy2: True, dummy2=None)
         start = time.time()
         while condition(start):
-            size = self._in_waiting()
-            if size:
+            avaialble = self._in_waiting()
+            if (more_data and not avaialble) or (more_data and avaialble == size):
                 break
+            if avaialble and avaialble != size:
+                more_data = True
+                size = avaialble
             time.sleep(0.01)
         return size
 
