@@ -8,6 +8,7 @@ import struct
 from pymodbus.pdu import ModbusRequest
 from pymodbus.pdu import ModbusResponse
 from pymodbus.pdu import ModbusExceptions as merror
+from pymodbus.compat import byte2int
 
 
 #---------------------------------------------------------------------------#
@@ -31,7 +32,8 @@ class FileRecord(object):
         self.file_number     = kwargs.get('file_number', 0x00)
         self.record_number   = kwargs.get('record_number', 0x00)
         self.record_data     = kwargs.get('record_data', '')
-        self.record_length   = kwargs.get('record_length',   len(self.record_data) / 2)
+
+        self.record_length   = kwargs.get('record_length',   len(self.record_data) // 2)
         self.response_length = kwargs.get('response_length', len(self.record_data) + 1)
 
     def __eq__(self, relf):
@@ -83,12 +85,12 @@ class ReadFileRecordRequest(ModbusRequest):
     function_code = 0x14
     _rtu_byte_count_pos = 2
 
-    def __init__(self, records=None):
+    def __init__(self, records=None, **kwargs):
         ''' Initializes a new instance
 
         :param records: The file record requests to be read
         '''
-        ModbusRequest.__init__(self)
+        ModbusRequest.__init__(self, **kwargs)
         self.records  = records or []
 
     def encode(self):
@@ -108,8 +110,8 @@ class ReadFileRecordRequest(ModbusRequest):
         :param data: The data to decode into the address
         '''
         self.records = []
-        byte_count = struct.unpack('B', data[0])[0]
-        for count in xrange(1, byte_count, 7):
+        byte_count = byte2int(data[0])
+        for count in range(1, byte_count, 7):
             decoded = struct.unpack('>BHHH', data[count:count+7])
             record  = FileRecord(file_number=decoded[1],
                 record_number=decoded[2], record_length=decoded[3])
@@ -138,12 +140,12 @@ class ReadFileRecordResponse(ModbusResponse):
     function_code = 0x14
     _rtu_byte_count_pos = 2
 
-    def __init__(self, records=None):
+    def __init__(self, records=None, **kwargs):
         ''' Initializes a new instance
 
         :param records: The requested file records
         '''
-        ModbusResponse.__init__(self)
+        ModbusResponse.__init__(self, **kwargs)
         self.records = records or []
 
     def encode(self):
@@ -164,7 +166,7 @@ class ReadFileRecordResponse(ModbusResponse):
         :param data: The packet data to decode
         '''
         count, self.records = 1, []
-        byte_count = struct.unpack('B', data[0])[0]
+        byte_count = byte2int(data[0])
         while count < byte_count:
             response_length, reference_type = struct.unpack('>BB', data[count:count+2])
             count += response_length + 1 # the count is not included
@@ -183,12 +185,12 @@ class WriteFileRecordRequest(ModbusRequest):
     function_code = 0x15
     _rtu_byte_count_pos = 2
 
-    def __init__(self, records=None):
+    def __init__(self, records=None, **kwargs):
         ''' Initializes a new instance
 
         :param records: The file record requests to be read
         '''
-        ModbusRequest.__init__(self)
+        ModbusRequest.__init__(self, **kwargs)
         self.records  = records or []
 
     def encode(self):
@@ -210,7 +212,7 @@ class WriteFileRecordRequest(ModbusRequest):
         :param data: The data to decode into the address
         '''
         count, self.records = 1, []
-        byte_count = struct.unpack('B', data[0])[0]
+        byte_count = byte2int(data[0])
         while count < byte_count:
             decoded = struct.unpack('>BHHH', data[count:count+7])
             response_length = decoded[3] * 2
@@ -239,12 +241,12 @@ class WriteFileRecordResponse(ModbusResponse):
     function_code = 0x15
     _rtu_byte_count_pos = 2
 
-    def __init__(self, records=None):
+    def __init__(self, records=None, **kwargs):
         ''' Initializes a new instance
 
         :param records: The file record requests to be read
         '''
-        ModbusResponse.__init__(self)
+        ModbusResponse.__init__(self, **kwargs)
         self.records  = records or []
 
     def encode(self):
@@ -266,7 +268,7 @@ class WriteFileRecordResponse(ModbusResponse):
         :param data: The data to decode into the address
         '''
         count, self.records = 1, []
-        byte_count = struct.unpack('B', data[0])[0]
+        byte_count = byte2int(data[0])
         while count < byte_count:
             decoded = struct.unpack('>BHHH', data[count:count+7])
             response_length = decoded[3] * 2
@@ -277,95 +279,6 @@ class WriteFileRecordResponse(ModbusResponse):
             if decoded[0] == 0x06: self.records.append(record)
 
 
-class MaskWriteRegisterRequest(ModbusRequest):
-    '''
-    This function code is used to modify the contents of a specified holding
-    register using a combination of an AND mask, an OR mask, and the
-    register's current contents. The function can be used to set or clear
-    individual bits in the register.
-    '''
-    function_code = 0x16
-    _rtu_frame_size = 10
-
-    def __init__(self, address=0x0000, and_mask=0xffff, or_mask=0x0000):
-        ''' Initializes a new instance
-
-        :param address: The mask pointer address (0x0000 to 0xffff)
-        :param and_mask: The and bitmask to apply to the register address
-        :param or_mask: The or bitmask to apply to the register address
-        '''
-        ModbusRequest.__init__(self)
-        self.address  = address
-        self.and_mask = and_mask
-        self.or_mask  = or_mask
-
-    def encode(self):
-        ''' Encodes the request packet
-
-        :returns: The byte encoded packet
-        '''
-        return struct.pack('>HHH', self.address, self.and_mask, self.or_mask)
-
-    def decode(self, data):
-        ''' Decodes the incoming request
-
-        :param data: The data to decode into the address
-        '''
-        self.address, self.and_mask, self.or_mask = struct.unpack('>HHH', data)
-
-    def execute(self, context):
-        ''' Run a mask write register request against the store
-
-        :param context: The datastore to request from
-        :returns: The populated response
-        '''
-        if not (0x0000 <= self.and_mask <= 0xffff):
-            return self.doException(merror.IllegalValue)
-        if not (0x0000 <= self.or_mask <= 0xffff):
-            return self.doException(merror.IllegalValue)
-        if not context.validate(self.function_code, self.address, 1):
-            return self.doException(merror.IllegalAddress)
-        values = context.getValues(self.function_code, self.address, 1)[0]
-        values = ((values & self.and_mask) | self.or_mask)
-        context.setValues(self.function_code, self.address, [values])
-        return MaskWriteRegisterResponse(self.address, self.and_mask, self.or_mask)
-
-
-class MaskWriteRegisterResponse(ModbusResponse):
-    '''
-    The normal response is an echo of the request. The response is returned
-    after the register has been written.
-    '''
-    function_code = 0x16
-    _rtu_frame_size = 10
-
-    def __init__(self, address=0x0000, and_mask=0xffff, or_mask=0x0000):
-        ''' Initializes a new instance
-
-        :param address: The mask pointer address (0x0000 to 0xffff)
-        :param and_mask: The and bitmask applied to the register address
-        :param or_mask: The or bitmask applied to the register address
-        '''
-        ModbusResponse.__init__(self)
-        self.address  = address
-        self.and_mask = and_mask
-        self.or_mask  = or_mask
-
-    def encode(self):
-        ''' Encodes the response
-
-        :returns: The byte encoded message
-        '''
-        return struct.pack('>HHH', self.address, self.and_mask, self.or_mask)
-
-    def decode(self, data):
-        ''' Decodes a the response
-
-        :param data: The packet data to decode
-        '''
-        self.address, self.and_mask, self.or_mask = struct.unpack('>HHH', data)
-
-
 class ReadFifoQueueRequest(ModbusRequest):
     '''
     This function code allows to read the contents of a First-In-First-Out
@@ -381,12 +294,12 @@ class ReadFifoQueueRequest(ModbusRequest):
     function_code = 0x18
     _rtu_frame_size = 6
 
-    def __init__(self, address=0x0000):
+    def __init__(self, address=0x0000, **kwargs):
         ''' Initializes a new instance
 
         :param address: The fifo pointer address (0x0000 to 0xffff)
         '''
-        ModbusRequest.__init__(self)
+        ModbusRequest.__init__(self, **kwargs)
         self.address = address
         self.values = []  # this should be added to the context
 
@@ -437,16 +350,16 @@ class ReadFifoQueueResponse(ModbusResponse):
         :param buffer: A buffer containing the data that have been received.
         :returns: The number of bytes in the response.
         '''
-        hi_byte = struct.unpack(">B", buffer[2])[0]
-        lo_byte = struct.unpack(">B", buffer[3])[0]
+        hi_byte = byte2int(buffer[2])
+        lo_byte = byte2int(buffer[3])
         return (hi_byte << 16) + lo_byte + 6
 
-    def __init__(self, values=None):
+    def __init__(self, values=None, **kwargs):
         ''' Initializes a new instance
 
         :param values: The list of values of the fifo to return
         '''
-        ModbusResponse.__init__(self)
+        ModbusResponse.__init__(self, **kwargs)
         self.values = values or []
 
     def encode(self):
@@ -466,8 +379,8 @@ class ReadFifoQueueResponse(ModbusResponse):
         :param data: The packet data to decode
         '''
         self.values = []
-        length, count = struct.unpack('>HH', data[0:4])
-        for index in xrange(0, count - 4):
+        _, count = struct.unpack('>HH', data[0:4])
+        for index in range(0, count - 4):
             idx = 4 + index * 2
             self.values.append(struct.unpack('>H', data[idx:idx + 2])[0])
 
@@ -478,6 +391,5 @@ __all__ = [
     "FileRecord",
     "ReadFileRecordRequest", "ReadFileRecordResponse",
     "WriteFileRecordRequest", "WriteFileRecordResponse",
-    "MaskWriteRegisterRequest", "MaskWriteRegisterResponse",
     "ReadFifoQueueRequest", "ReadFifoQueueResponse",
 ]
