@@ -14,7 +14,7 @@ from pymodbus.utilities import pack_bitstring
 from pymodbus.utilities import unpack_bitstring
 from pymodbus.utilities import make_byte_string
 from pymodbus.exceptions import ParameterException
-
+from pymodbus.compat import unicode_string
 # --------------------------------------------------------------------------- #
 # Logging
 # --------------------------------------------------------------------------- #
@@ -124,6 +124,17 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         _logger.debug(payload)
         return payload
 
+    def to_coils(self):
+        """Convert the payload buffer into a coil
+        layout that can be used as a context block.
+
+        :returns: The coil layout to use as a block
+        """
+        payload = self.to_registers()
+        coils = [bool(int(bit)) for reg
+                 in payload[1:] for bit in format(reg, '016b')]
+        return coils
+
     def build(self):
         """ Return the payload buffer as a list
 
@@ -320,7 +331,7 @@ class BinaryPayloadDecoder(object):
         :return:
         """
         handle = make_byte_string(handle)
-        wc = WC.get(fstring.lower())//2
+        wc = WC.get(fstring.lower()) // 2
         up = "!{}H".format(wc)
         handle = unpack(up, handle)
         if self._wordorder == Endian.Little:
@@ -329,8 +340,8 @@ class BinaryPayloadDecoder(object):
         # Repack as unsigned Integer
         pk = self._byteorder + 'H'
         handle = [pack(pk, p) for p in handle]
-        handle = b''.join(handle)
         _logger.debug(handle)
+        handle = b''.join(handle)
         return handle
 
     def reset(self):
