@@ -103,15 +103,24 @@ class ModbusPayloadUtilityTests(unittest.TestCase):
 
     def testPayloadBuilderWithRawPayload(self):
         """ Test basic bit message encoding/decoding """
-        builder = BinaryPayloadBuilder([b'\x12', b'\x34', b'\x56', b'\x78'], repack=True)
+        _coils1 = [False, True, True, True, True, False, False, False, False,
+                   True, False, True, False, True, True, False]
+        _coils2 = [False, True, False, True, False, True, True, False,
+                   False, True, True, True, True, False, False, False]
+
+        builder = BinaryPayloadBuilder([b'\x12', b'\x34', b'\x56', b'\x78'],
+                                       repack=True)
         self.assertEqual(b'\x12\x34\x56\x78', builder.to_string())
         self.assertEqual([13330, 30806], builder.to_registers())
 
+        self.assertEqual(_coils1, builder.to_coils())
+
         builder = BinaryPayloadBuilder([b'\x12', b'\x34', b'\x56', b'\x78'],
                                        byteorder=Endian.Big)
         self.assertEqual(b'\x12\x34\x56\x78', builder.to_string())
         self.assertEqual([4660, 22136], builder.to_registers())
         self.assertEqual('\x12\x34\x56\x78', str(builder))
+        self.assertEqual(_coils2, builder.to_coils())
 
     # ----------------------------------------------------------------------- #
     # Payload Decoder Tests
