@@ -5,8 +5,9 @@ from pymodbus.register_read_message import ReadRegistersRequestBase
 from pymodbus.register_read_message import ReadRegistersResponseBase
 from pymodbus.exceptions import *
 from pymodbus.pdu import ModbusExceptions
+from pymodbus.compat import iteritems, iterkeys, get_next
 
-from modbus_mocks import MockContext, FakeList
+from .modbus_mocks import MockContext, FakeList
 
 #---------------------------------------------------------------------------#
 # Fixture
@@ -34,17 +35,17 @@ class ReadRegisterMessagesTest(unittest.TestCase):
         self.value  = 0xabcd
         self.values = [0xa, 0xb, 0xc]
         self.request_read  = {
-            ReadRegistersRequestBase(1, 5)                  :'\x00\x01\x00\x05',
-            ReadHoldingRegistersRequest(1, 5)               :'\x00\x01\x00\x05',
-            ReadInputRegistersRequest(1,5)                  :'\x00\x01\x00\x05',
-            ReadWriteMultipleRegistersRequest(**arguments)  :'\x00\x01\x00\x05\x00\x01\x00'
-                                                             '\x05\x0a\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
+            ReadRegistersRequestBase(1, 5)                  :b'\x00\x01\x00\x05',
+            ReadHoldingRegistersRequest(1, 5)               :b'\x00\x01\x00\x05',
+            ReadInputRegistersRequest(1,5)                  :b'\x00\x01\x00\x05',
+            ReadWriteMultipleRegistersRequest(**arguments)  :b'\x00\x01\x00\x05\x00\x01\x00'
+                                                             b'\x05\x0a\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
         }
         self.response_read  = {
-            ReadRegistersResponseBase(self.values)          :'\x06\x00\x0a\x00\x0b\x00\x0c',
-            ReadHoldingRegistersResponse(self.values)       :'\x06\x00\x0a\x00\x0b\x00\x0c',
-            ReadInputRegistersResponse(self.values)         :'\x06\x00\x0a\x00\x0b\x00\x0c',
-            ReadWriteMultipleRegistersResponse(self.values) :'\x06\x00\x0a\x00\x0b\x00\x0c',
+            ReadRegistersResponseBase(self.values)          :b'\x06\x00\x0a\x00\x0b\x00\x0c',
+            ReadHoldingRegistersResponse(self.values)       :b'\x06\x00\x0a\x00\x0b\x00\x0c',
+            ReadInputRegistersResponse(self.values)         :b'\x06\x00\x0a\x00\x0b\x00\x0c',
+            ReadWriteMultipleRegistersResponse(self.values) :b'\x06\x00\x0a\x00\x0b\x00\x0c',
         }
 
     def tearDown(self):
@@ -53,16 +54,16 @@ class ReadRegisterMessagesTest(unittest.TestCase):
         del self.response_read
 
     def testReadRegisterResponseBase(self):
-        response = ReadRegistersResponseBase(range(10))
+        response = ReadRegistersResponseBase(list(range(10)))
         for index in range(10):
             self.assertEqual(response.getRegister(index), index)
 
     def testRegisterReadRequests(self):
-        for request, response in self.request_read.items():
+        for request, response in iteritems(self.request_read):
             self.assertEqual(request.encode(), response)
 
     def testRegisterReadResponses(self):
-        for request, response in self.response_read.items():
+        for request, response in iteritems(self.response_read):
             self.assertEqual(request.encode(), response)
 
     def testRegisterReadResponseDecode(self):
@@ -72,7 +73,7 @@ class ReadRegisterMessagesTest(unittest.TestCase):
             [0x0a,0x0b,0x0c],
             [0x0a,0x0b,0x0c, 0x0a,0x0b,0x0c],
         ]
-        values = sorted(self.response_read.items())
+        values = sorted(self.response_read.items(), key=lambda x: str(x))
         for packet, register in zip(values, registers):
             request, response = packet
             request.decode(response)
@@ -152,7 +153,8 @@ class ReadRegisterMessagesTest(unittest.TestCase):
         self.assertEqual(response.exception_code, ModbusExceptions.IllegalValue)
 
     def testReadWriteMultipleRegistersRequestDecode(self):
-        request, response = sorted(self.request_read.items())[-1]
+        request, response = get_next((k,v) for k,v in self.request_read.items()
+            if getattr(k, 'function_code', 0) == 23)
         request.decode(response)
         self.assertEqual(request.read_address, 0x01)
         self.assertEqual(request.write_address, 0x01)
@@ -162,9 +164,9 @@ class ReadRegisterMessagesTest(unittest.TestCase):
         self.assertEqual(request.write_registers, [0x00]*5)
 
     def testSerializingToString(self):
-        for request in self.request_read.keys():
+        for request in iterkeys(self.request_read):
             self.assertTrue(str(request) != None)
-        for request in self.response_read.keys():
+        for request in iterkeys(self.response_read):
             self.assertTrue(str(request) != None)
 
 #---------------------------------------------------------------------------#
