@@ -1,49 +1,142 @@
 #!/usr/bin/env python
+import pytest
 import unittest
 from binascii import a2b_hex
 from pymodbus.pdu import *
 from pymodbus.transaction import *
+from pymodbus.transaction import (
+    ModbusTransactionManager, ModbusSocketFramer, ModbusAsciiFramer,
+    ModbusRtuFramer, ModbusBinaryFramer
+)
 from pymodbus.factory import ServerDecoder
 from pymodbus.compat import byte2int
+from mock import MagicMock
+from pymodbus.exceptions import (
+    NotImplementedException, ModbusIOException, InvalidMessageReceivedException
+)
+
 
 class ModbusTransactionTest(unittest.TestCase):
-    '''
+    """
     This is the unittest for the pymodbus.transaction module
-    '''
+    """
 
-    #---------------------------------------------------------------------------#
+    # ----------------------------------------------------------------------- #
     # Test Construction
-    #---------------------------------------------------------------------------#
+    # ----------------------------------------------------------------------- #
     def setUp(self):
-        ''' Sets up the test environment '''
+        """ Sets up the test environment """
         self.client   = None
         self.decoder  = ServerDecoder()
-        self._tcp     = ModbusSocketFramer(decoder=self.decoder)
-        self._rtu     = ModbusRtuFramer(decoder=self.decoder)
-        self._ascii   = ModbusAsciiFramer(decoder=self.decoder)
-        self._binary  = ModbusBinaryFramer(decoder=self.decoder)
+        self._tcp     = ModbusSocketFramer(decoder=self.decoder, client=None)
+        self._rtu     = ModbusRtuFramer(decoder=self.decoder, client=None)
+        self._ascii   = ModbusAsciiFramer(decoder=self.decoder, client=None)
+        self._binary  = ModbusBinaryFramer(decoder=self.decoder, client=None)
         self._manager = DictTransactionManager(self.client)
         self._queue_manager = FifoTransactionManager(self.client)
+        self._tm = ModbusTransactionManager(self.client)
 
     def tearDown(self):
-        ''' Cleans up the test environment '''
+        """ Cleans up the test environment """
         del self._manager
         del self._tcp
         del self._rtu
         del self._ascii
 
-    #---------------------------------------------------------------------------# 
+    # ----------------------------------------------------------------------- #
+    # Base transaction manager
+    # ----------------------------------------------------------------------- #
+
+    def testCalculateExpectedResponseLength(self):
+        self._tm.client = MagicMock()
+        self._tm.client.framer = MagicMock()
+        self._tm._set_adu_size()
+        self.assertEqual(self._tm._calculate_response_length(0), None)
+        self._tm.base_adu_size = 10
+        self.assertEqual(self._tm._calculate_response_length(5), 15)
+
+    def testCalculateExceptionLength(self):
+        for framer, exception_length in [('ascii', 11),
+                                         ('binary', 7),
+                                         ('rtu', 5),
+                                         ('tcp', 9),
+                                         ('dummy', None)]:
+            self._tm.client = MagicMock()
+            if framer == "ascii":
+                self._tm.client.framer = self._ascii
+            elif framer == "binary":
+                self._tm.client.framer = self._binary
+            elif framer == "rtu":
+                self._tm.client.framer = self._rtu
+            elif framer == "tcp":
+                self._tm.client.framer = self._tcp
+            else:
+                self._tm.client.framer = MagicMock()
+
+            self._tm._set_adu_size()
+            self.assertEqual(self._tm._calculate_exception_length(),
+                             exception_length)
+
+    def testExecute(self):
+        client = MagicMock()
+        client.framer = self._ascii
+        client.framer._buffer = b'deadbeef'
+        client.framer.processIncomingPacket = MagicMock()
+        client.framer.processIncomingPacket.return_value = None
+        client.framer.buildPacket = MagicMock()
+        client.framer.buildPacket.return_value = b'deadbeef'
+        client.framer.sendPacket = MagicMock()
+        client.framer.sendPacket.return_value = len(b'deadbeef')
+
+        request = MagicMock()
+        request.get_response_pdu_size.return_value = 10
+        request.unit_id = 1
+        tm = ModbusTransactionManager(client)
+        tm._recv = MagicMock(return_value=b'abcdef')
+        self.assertEqual(tm.retries, 3)
+        self.assertEqual(tm.retry_on_empty, False)
+        # tm._transact = MagicMock()
+        # some response
+        # tm._transact.return_value = (b'abcdef', None)
+        tm.getTransaction = MagicMock()
+        tm.getTransaction.return_value = 'response'
+        response = tm.execute(request)
+        self.assertEqual(response, 'response')
+        # No response
+        tm._recv = MagicMock(return_value=b'abcdef')
+        # tm._transact.return_value = (b'', None)
+        tm.transactions = []
+        tm.getTransaction = MagicMock()
+        tm.getTransaction.return_value = None
+        response = tm.execute(request)
+        self.assertIsInstance(response, ModbusIOException)
+
+        # No response with retries
+        tm.retry_on_empty = True
+        tm._recv = MagicMock(side_effect=iter([b'', b'abcdef']))
+        # tm._transact.side_effect = [(b'', None), (b'abcdef', None)]
+        response = tm.execute(request)
+        self.assertIsInstance(response, ModbusIOException)
+
+        # Unable to decode response
+        tm._recv = MagicMock(side_effect=ModbusIOException())
+        # tm._transact.side_effect = [(b'abcdef', None)]
+        client.framer.processIncomingPacket.side_effect = MagicMock(side_effect=ModbusIOException())
+        self.assertIsInstance(tm.execute(request), ModbusIOException)
+
+    # ----------------------------------------------------------------------- #
     # Dictionary based transaction manager
-    #---------------------------------------------------------------------------# 
+    # ----------------------------------------------------------------------- #
+
     def testDictTransactionManagerTID(self):
-        ''' Test the dict transaction manager TID '''
+        """ Test the dict transaction manager TID """
         for tid in range(1, self._manager.getNextTID() + 10):
             self.assertEqual(tid+1, self._manager.getNextTID())
         self._manager.reset()
         self.assertEqual(1, self._manager.getNextTID())
 
     def testGetDictTransactionManagerTransaction(self):
-        ''' Test the dict transaction manager '''
+        """ Test the dict transaction manager """
         class Request: pass
         self._manager.reset()
         handle = Request()
@@ -54,7 +147,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(handle.message, result.message)
 
     def testDeleteDictTransactionManagerTransaction(self):
-        ''' Test the dict transaction manager '''
+        """ Test the dict transaction manager """
         class Request: pass
         self._manager.reset()
         handle = Request()
@@ -65,18 +158,18 @@ class ModbusTransactionTest(unittest.TestCase):
         self._manager.delTransaction(handle.transaction_id)
         self.assertEqual(None, self._manager.getTransaction(handle.transaction_id))
 
-    #---------------------------------------------------------------------------# 
+    # ----------------------------------------------------------------------- #
     # Queue based transaction manager
-    #---------------------------------------------------------------------------# 
+    # ----------------------------------------------------------------------- #
     def testFifoTransactionManagerTID(self):
-        ''' Test the fifo transaction manager TID '''
+        """ Test the fifo transaction manager TID """
         for tid in range(1, self._queue_manager.getNextTID() + 10):
             self.assertEqual(tid+1, self._queue_manager.getNextTID())
         self._queue_manager.reset()
         self.assertEqual(1, self._queue_manager.getNextTID())
 
     def testGetFifoTransactionManagerTransaction(self):
-        ''' Test the fifo transaction manager '''
+        """ Test the fifo transaction manager """
         class Request: pass
         self._queue_manager.reset()
         handle = Request()
@@ -87,7 +180,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(handle.message, result.message)
 
     def testDeleteFifoTransactionManagerTransaction(self):
-        ''' Test the fifo transaction manager '''
+        """ Test the fifo transaction manager """
         class Request: pass
         self._queue_manager.reset()
         handle = Request()
@@ -98,11 +191,11 @@ class ModbusTransactionTest(unittest.TestCase):
         self._queue_manager.delTransaction(handle.transaction_id)
         self.assertEqual(None, self._queue_manager.getTransaction(handle.transaction_id))
 
-    #---------------------------------------------------------------------------# 
+    # ----------------------------------------------------------------------- #
     # TCP tests
-    #---------------------------------------------------------------------------#
+    # ----------------------------------------------------------------------- #
     def testTCPFramerTransactionReady(self):
-        ''' Test a tcp frame transaction '''
+        """ Test a tcp frame transaction """
         msg = b"\x00\x01\x12\x34\x00\x04\xff\x02\x12\x34"
         self.assertFalse(self._tcp.isFrameReady())
         self.assertFalse(self._tcp.checkFrame())
@@ -115,7 +208,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(b'', self._ascii.getFrame())
 
     def testTCPFramerTransactionFull(self):
-        ''' Test a full tcp frame transaction '''
+        """ Test a full tcp frame transaction """
         msg = b"\x00\x01\x12\x34\x00\x04\xff\x02\x12\x34"
         self._tcp.addToFrame(msg)
         self.assertTrue(self._tcp.checkFrame())
@@ -124,7 +217,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self._tcp.advanceFrame()
 
     def testTCPFramerTransactionHalf(self):
-        ''' Test a half completed tcp frame transaction '''
+        """ Test a half completed tcp frame transaction """
         msg1 = b"\x00\x01\x12\x34\x00"
         msg2 = b"\x04\xff\x02\x12\x34"
         self._tcp.addToFrame(msg1)
@@ -138,7 +231,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self._tcp.advanceFrame()
 
     def testTCPFramerTransactionHalf2(self):
-        ''' Test a half completed tcp frame transaction '''
+        """ Test a half completed tcp frame transaction """
         msg1 = b"\x00\x01\x12\x34\x00\x04\xff"
         msg2 = b"\x02\x12\x34"
         self._tcp.addToFrame(msg1)
@@ -152,7 +245,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self._tcp.advanceFrame()
 
     def testTCPFramerTransactionHalf3(self):
-        ''' Test a half completed tcp frame transaction '''
+        """ Test a half completed tcp frame transaction """
         msg1 = b"\x00\x01\x12\x34\x00\x04\xff\x02\x12"
         msg2 = b"\x34"
         self._tcp.addToFrame(msg1)
@@ -166,7 +259,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self._tcp.advanceFrame()
 
     def testTCPFramerTransactionShort(self):
-        ''' Test that we can get back on track after an invalid message '''
+        """ Test that we can get back on track after an invalid message """
         msg1 = b"\x99\x99\x99\x99\x00\x01\x00\x01"
         msg2 = b"\x00\x01\x12\x34\x00\x04\xff\x02\x12\x34"
         self._tcp.addToFrame(msg1)
@@ -175,14 +268,14 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(b'', result)
         self._tcp.advanceFrame()
         self._tcp.addToFrame(msg2)
-        self.assertEqual(10, len(self._tcp._ModbusSocketFramer__buffer))
+        self.assertEqual(10, len(self._tcp._buffer))
         self.assertTrue(self._tcp.checkFrame())
         result = self._tcp.getFrame()
         self.assertEqual(msg2[7:], result)
         self._tcp.advanceFrame()
 
     def testTCPFramerPopulate(self):
-        ''' Test a tcp frame packet build '''
+        """ Test a tcp frame packet build """
         expected = ModbusRequest()
         expected.transaction_id = 0x0001
         expected.protocol_id    = 0x1234
@@ -197,7 +290,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self._tcp.advanceFrame()
 
     def testTCPFramerPacket(self):
-        ''' Test a tcp frame packet build '''
+        """ Test a tcp frame packet build """
         old_encode = ModbusRequest.encode
         ModbusRequest.encode = lambda self: b''
         message = ModbusRequest()
@@ -210,11 +303,11 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(expected, actual)
         ModbusRequest.encode = old_encode
 
-    #---------------------------------------------------------------------------#
+    # ----------------------------------------------------------------------- #
     # RTU tests
-    #---------------------------------------------------------------------------#
+    # ----------------------------------------------------------------------- #
     def testRTUFramerTransactionReady(self):
-        ''' Test if the checks for a complete frame work '''
+        """ Test if the checks for a complete frame work """
         self.assertFalse(self._rtu.isFrameReady())
 
         msg_parts = [b"\x00\x01\x00", b"\x00\x00\x01\xfc\x1b"]
@@ -227,7 +320,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertTrue(self._rtu.checkFrame())
 
     def testRTUFramerTransactionFull(self):
-        ''' Test a full rtu frame transaction '''
+        """ Test a full rtu frame transaction """
         msg = b"\x00\x01\x00\x00\x00\x01\xfc\x1b"
         stripped_msg = msg[1:-2]
         self._rtu.addToFrame(msg)
@@ -237,7 +330,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self._rtu.advanceFrame()
 
     def testRTUFramerTransactionHalf(self):
-        ''' Test a half completed rtu frame transaction '''
+        """ Test a half completed rtu frame transaction """
         msg_parts = [b"\x00\x01\x00", b"\x00\x00\x01\xfc\x1b"]
         stripped_msg = b"".join(msg_parts)[1:-2]
         self._rtu.addToFrame(msg_parts[0])
@@ -250,14 +343,14 @@ class ModbusTransactionTest(unittest.TestCase):
         self._rtu.advanceFrame()
 
     def testRTUFramerPopulate(self):
-        ''' Test a rtu frame packet build '''
+        """ Test a rtu frame packet build """
         request = ModbusRequest()
         msg = b"\x00\x01\x00\x00\x00\x01\xfc\x1b"
         self._rtu.addToFrame(msg)
         self._rtu.populateHeader()
         self._rtu.populateResult(request)
 
-        header_dict = self._rtu._ModbusRtuFramer__header
+        header_dict = self._rtu._header
         self.assertEqual(len(msg), header_dict['len'])
         self.assertEqual(byte2int(msg[0]), header_dict['uid'])
         self.assertEqual(msg[-2:], header_dict['crc'])
@@ -265,7 +358,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(0x00, request.unit_id)
 
     def testRTUFramerPacket(self):
-        ''' Test a rtu frame packet build '''
+        """ Test a rtu frame packet build """
         old_encode = ModbusRequest.encode
         ModbusRequest.encode = lambda self: b''
         message = ModbusRequest()
@@ -277,17 +370,54 @@ class ModbusTransactionTest(unittest.TestCase):
         ModbusRequest.encode = old_encode
 
     def testRTUDecodeException(self):
-        ''' Test that the RTU framer can decode errors '''
+        """ Test that the RTU framer can decode errors """
         message = b"\x00\x90\x02\x9c\x01"
         actual = self._rtu.addToFrame(message)
         result = self._rtu.checkFrame()
         self.assertTrue(result)
 
-    #---------------------------------------------------------------------------#
+    def testProcess(self):
+        class MockResult(object):
+            def __init__(self, code):
+                self.function_code = code
+
+        def mock_callback(self):
+            pass
+
+        mock_result = MockResult(code=0)
+        self._rtu.getRawFrame = self._rtu.getFrame = MagicMock()
+        self._rtu.decoder = MagicMock()
+        self._rtu.decoder.decode = MagicMock(return_value=mock_result)
+        self._rtu.populateResult = MagicMock()
+        self._rtu.advanceFrame = MagicMock()
+
+        self._rtu._process(mock_callback)
+        self._rtu.populateResult.assert_called_with(mock_result)
+        self._rtu.advanceFrame.assert_called_with()
+        self.assertTrue(self._rtu.advanceFrame.called)
+
+        #Check errors
+        self._rtu.decoder.decode = MagicMock(return_value=None)
+        self.assertRaises(ModbusIOException, lambda: self._rtu._process(mock_callback))
+
+    def testRTUProcessIncomingPAkcets(self):
+        mock_data = b"\x00\x01\x00\x00\x00\x01\xfc\x1b"
+        unit = 0x00
+        def mock_callback(self):
+            pass
+
+        self._rtu.addToFrame = MagicMock()
+        self._rtu._process = MagicMock()
+        self._rtu.isFrameReady = MagicMock(return_value=False)
+        self._rtu._buffer = mock_data
+
+        self._rtu.processIncomingPacket(mock_data, mock_callback, unit)
+
+    # ----------------------------------------------------------------------- #
     # ASCII tests
-    #---------------------------------------------------------------------------#
+    # ----------------------------------------------------------------------- #
     def testASCIIFramerTransactionReady(self):
-        ''' Test a ascii frame transaction '''
+        """ Test a ascii frame transaction """
         msg = b':F7031389000A60\r\n'
         self.assertFalse(self._ascii.isFrameReady())
         self.assertFalse(self._ascii.checkFrame())
@@ -300,7 +430,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(b'', self._ascii.getFrame())
 
     def testASCIIFramerTransactionFull(self):
-        ''' Test a full ascii frame transaction '''
+        """ Test a full ascii frame transaction """
         msg = b'sss:F7031389000A60\r\n'
         pack = a2b_hex(msg[6:-4])
         self._ascii.addToFrame(msg)
@@ -310,7 +440,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self._ascii.advanceFrame()
 
     def testASCIIFramerTransactionHalf(self):
-        ''' Test a half completed ascii frame transaction '''
+        """ Test a half completed ascii frame transaction """
         msg1 = b'sss:F7031389'
         msg2 = b'000A60\r\n'
         pack = a2b_hex(msg1[6:] + msg2[:-4])
@@ -325,13 +455,13 @@ class ModbusTransactionTest(unittest.TestCase):
         self._ascii.advanceFrame()
 
     def testASCIIFramerPopulate(self):
-        ''' Test a ascii frame packet build '''
+        """ Test a ascii frame packet build """
         request = ModbusRequest()
         self._ascii.populateResult(request)
         self.assertEqual(0x00, request.unit_id)
 
     def testASCIIFramerPacket(self):
-        ''' Test a ascii frame packet build '''
+        """ Test a ascii frame packet build """
         old_encode = ModbusRequest.encode
         ModbusRequest.encode = lambda self: b''
         message = ModbusRequest()
@@ -342,11 +472,23 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(expected, actual)
         ModbusRequest.encode = old_encode
 
-    #---------------------------------------------------------------------------#
+    def testAsciiProcessIncomingPakcets(self):
+        mock_data = msg = b':F7031389000A60\r\n'
+        unit = 0x00
+        def mock_callback(mock_data, *args, **kwargs):
+            pass
+
+        self._ascii.processIncomingPacket(mock_data, mock_callback, unit)
+
+        # Test failure:
+        self._ascii.checkFrame = MagicMock(return_value=False)
+        self._ascii.processIncomingPacket(mock_data, mock_callback, unit)
+
+    # ----------------------------------------------------------------------- #
     # Binary tests
-    #---------------------------------------------------------------------------#
+    # ----------------------------------------------------------------------- #
     def testBinaryFramerTransactionReady(self):
-        ''' Test a binary frame transaction '''
+        """ Test a binary frame transaction """
         msg  = b'\x7b\x01\x03\x00\x00\x00\x05\x85\xC9\x7d'
         self.assertFalse(self._binary.isFrameReady())
         self.assertFalse(self._binary.checkFrame())
@@ -359,9 +501,9 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(b'', self._binary.getFrame())
 
     def testBinaryFramerTransactionFull(self):
-        ''' Test a full binary frame transaction '''
+        """ Test a full binary frame transaction """
         msg  = b'\x7b\x01\x03\x00\x00\x00\x05\x85\xC9\x7d'
-        pack = msg[3:-3]
+        pack = msg[2:-3]
         self._binary.addToFrame(msg)
         self.assertTrue(self._binary.checkFrame())
         result = self._binary.getFrame()
@@ -369,10 +511,10 @@ class ModbusTransactionTest(unittest.TestCase):
         self._binary.advanceFrame()
 
     def testBinaryFramerTransactionHalf(self):
-        ''' Test a half completed binary frame transaction '''
+        """ Test a half completed binary frame transaction """
         msg1 = b'\x7b\x01\x03\x00'
         msg2 = b'\x00\x00\x05\x85\xC9\x7d'
-        pack = msg1[3:] + msg2[:-3]
+        pack = msg1[2:] + msg2[:-3]
         self._binary.addToFrame(msg1)
         self.assertFalse(self._binary.checkFrame())
         result = self._binary.getFrame()
@@ -384,13 +526,13 @@ class ModbusTransactionTest(unittest.TestCase):
         self._binary.advanceFrame()
 
     def testBinaryFramerPopulate(self):
-        ''' Test a binary frame packet build '''
+        """ Test a binary frame packet build """
         request = ModbusRequest()
         self._binary.populateResult(request)
         self.assertEqual(0x00, request.unit_id)
 
     def testBinaryFramerPacket(self):
-        ''' Test a binary frame packet build '''
+        """ Test a binary frame packet build """
         old_encode = ModbusRequest.encode
         ModbusRequest.encode = lambda self: b''
         message = ModbusRequest()
@@ -401,8 +543,22 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(expected, actual)
         ModbusRequest.encode = old_encode
 
-#---------------------------------------------------------------------------#
+    def testBinaryProcessIncomingPacket(self):
+        mock_data = b'\x7b\x01\x03\x00\x00\x00\x05\x85\xC9\x7d'
+        unit = 0x00
+        def mock_callback(mock_data):
+            pass
+
+        self._binary.processIncomingPacket(mock_data, mock_callback, unit)
+
+        # Test failure:
+        self._binary.checkFrame = MagicMock(return_value=False)
+        self._binary.processIncomingPacket(mock_data, mock_callback, unit)
+
+# ----------------------------------------------------------------------- #
 # Main
-#---------------------------------------------------------------------------#
+# ----------------------------------------------------------------------- #
+
+
 if __name__ == "__main__":
-    unittest.main()
+    pytest.main()
